from django.contrib import admin
from django.contrib.flatpages.models import FlatPage
from django.utils.translation import gettext_lazy as _
from django.utils.safestring import mark_safe
from .models import Page, Attachment


class PageInLine(admin.TabularInline):
    model = Page
    list_display=['has_attachments']


class AttachmentInline(admin.TabularInline):
    model = Attachment
    extra = 0


class FlatPageAdmin(admin.ModelAdmin):
    inlines = [PageInLine, AttachmentInline]
    fields = [
        'title',
        'url',
        'content',
        'template_name',
        'registration_required',
        'sites',
    ]
    list_display = (
        'title',
        'url',
        'registration_required',
        'page',
        'get_sites',
    )
    list_filter = (
        'sites',
        'registration_required',
    )
    filter_horizontal = ('sites',)

    @admin.display(description=_('Sites'))
    def get_sites(self, obj):
        return mark_safe(
            '<br>'.join([f'{x.name} ({x.domain})' for x in obj.sites.all()]))


admin.site.unregister(FlatPage)
admin.site.register(FlatPage, FlatPageAdmin)



from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.flatpages.models import FlatPage
from django.dispatch import receiver
from django.db.models.signals import pre_save
from .managers import PageManager


class Attachment(models.Model):
    flatpage = models.ForeignKey(
        FlatPage,
        related_name='attachments',
        on_delete=models.CASCADE,
    )
    title = models.CharField(
        max_length=250,
    )
    file = models.FileField(
        _('File'),
        upload_to='page_attachments/'
    )

    def __str__(self):
        return self.file.name

    class Meta:
        verbose_name = _('Attachment')
        verbose_name_plural = _('Attachments')


class Category(models.IntegerChoices):
    """
    COMPANY -> Used in footer middle
    LEGAL -> Used in footer right
    """
    UNDEFINED = 0, _('Undefined')
    COMPANY = 1, _('Company')
    LEGAL = 2, _('Legal')


class Page(models.Model):
    """ This is passed to the template context to build menu and urls """
    flatpage = models.OneToOneField(
        FlatPage,
        related_name='page',
        on_delete=models.CASCADE,
    )
    category = models.PositiveSmallIntegerField(
        _('Category'),
        choices=Category.choices,
        default=Category.UNDEFINED,
    )
    priority = models.SmallIntegerField(
        _('Priority'),
        default=0,
    )
    objects = PageManager()

    @property
    def title(self):
        return self.flatpage.title

    @property
    def category_name(self):
        return Category(self.category).label

    def get_absolute_url(self):
        return self.flatpage.get_absolute_url()

    def __str__(self):
        return f'{self.category_name}: {self.flatpage}'

    class Meta:
        ordering = ['category', 'priority']
        verbose_name = _('Categorized page')
        verbose_name_plural = _('Categorized pages')


@receiver(pre_save, sender=FlatPage)
def flatpage_pre_save(sender, instance, **kwargs):
    if not instance.url.startswith('/'):
        instance.url = '/' + instance.url
    if not instance.url.endswith('/'):
        instance.url += '/'





SECRET_KEY = 'notverysecret'


INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    'django.contrib.flatpages',
    'django.contrib.messages',
    'pages.apps.PagesConfig',
]


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'mydatabase',
    }
}


MIDDLEWARE = [
    #'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    #'django.middleware.common.CommonMiddleware',
    #'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    #'django.middleware.clickjacking.XFrameOptionsMiddleware',
]


TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

ROOT_URLCONF = 'urls'
DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'



import os
from pathlib import Path


def get_version():
    path = Path(str(Path(__file__).parent.parent.parent) + os.sep + 'VERSION')
    if path.is_file():
        with open(path) as f:
            return f.read().strip()
    return '0.0.0'


__version__ = get_version()



# Generated by Django 3.1.6 on 2021-06-09 18:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='page',
            options={'ordering': ['category', '-priority'], 'verbose_name': 'Categorized page', 'verbose_name_plural': 'Categorized pages'},
        ),
    ]



# Generated by Django 3.1.6 on 2021-06-21 06:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0002_auto_20210609_2039'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='page',
            options={'ordering': ['category', 'priority'], 'verbose_name': 'Categorized page', 'verbose_name_plural': 'Categorized pages'},
        ),
        migrations.AddField(
            model_name='attachment',
            name='title',
            field=models.CharField(default='', max_length=250),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.1.6 on 2021-06-09 17:39
import os
from pathlib import Path

from django.db import migrations, models
import django.db.models.deletion
from django.core.management import call_command


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('flatpages', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('category', models.PositiveSmallIntegerField(choices=[(0, 'Undefined'), (1, 'Company'), (2, 'Legal')], default=0, verbose_name='Category')),
                ('priority', models.SmallIntegerField(default=0, verbose_name='Priority')),
                ('flatpage', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='page', to='flatpages.flatpage')),
            ],
            options={
                'verbose_name': 'Categorized page',
                'verbose_name_plural': 'Categorized pages',
                'ordering': ['-priority'],
            },
        ),
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(upload_to='page_attachments/', verbose_name='File')),
                ('flatpage', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='flatpages.flatpage')),
            ],
            options={
                'verbose_name': 'Attachment',
                'verbose_name_plural': 'Attachments',
            },
        ),
    ]



from django.test import TestCase
from django.core.files import File
from django.contrib.flatpages.models import FlatPage
from pages.models import Page, Category, Attachment

class TestPageModel(TestCase):
    fixtures = ['initial_pages']

    def setUp(self):
        self.flatpage = FlatPage.objects.create()
        self.page = Page.objects.create(
            flatpage=self.flatpage,
            category=Category.COMPANY,
            priority=10,
        )
    
    def test_str(self):
        self.assertIn(str(self.page.category_name), str(self.page))
        #breakpoint()

    def test_url(self):
        self.assertEqual(
            self.page.get_absolute_url(),
            self.flatpage.get_absolute_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.page.title,
            self.flatpage.title,
        )

class TestAttachmentModel(TestCase):
    def setUp(self):
        self.flatpage = FlatPage.objects.create()
        self.file = File(file=b"", name='test.pdf')
        self.attachment = Attachment(
            file=self.file,
            title='testtitle',
            flatpage=self.flatpage,
        )


    def test_str(self):
        self.assertEqual(str(self.attachment), self.file.name)
